<?php
namespace Teknisa\CADEST\Controllers;

use Doctrine\ORM\EntityManager;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Teknisa\CADEST\Controllers\TeknisaCrud;
use Teknisa\CADEST\Services\Ger01700UnidadeService;
use Teknisa\Libs\Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Ger01700UnidadeController extends TeknisaCrud {
    protected $dataSourceNames = array(
        'default'      => '',
        'filial'       => 'filial_view',
        'endefili'     => 'endefili_view',
        'almoxarifado' => 'almoxarifado',
        'localestoque' => 'localestoque',
        'paramfilial'  => 'paramfilial',
        'filipadr'     => 'paramfilipadr',
        'filioper'     => 'filioper',
        'paramgeral'   => 'paramgeral'
    );

    protected $entityManager;
    protected $environment;
    protected $service;
    protected $cripto;
    protected $loginUrl;
    protected $clientSecret;
    protected $clientId;
    protected $productId;
    protected $nrorg;

    /**
     * Construtor do Controller.
     *
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param EntityManager $entityManager
     * @param Environment $environment
     * @param Ger01700UnidadeService $service
     */
    public function __construct($crudService, ParameterBag $parameterBag, EntityManager $entityManager, $environment, $cripto, Ger01700UnidadeService $service, $cpUrl) {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->service       = $service;
        $this->cripto        = $cripto;
        $this->cpUrl          = $cpUrl;

        $this->switchDataSource('default');
        
        // __DIR__ = /home/developer/workfolder/erp/modules/cadEst/backend/src/Teknisa/CADEST/Controllers
        $config = json_decode(file_get_contents(__DIR__ . '/../../../../vendor/teknisa/libraries/config/config.json'), true);
        $integration = json_decode(file_get_contents(__DIR__ . '/../../../../vendor/teknisa/libraries/config/integration.json'), true);
        // $this->loginUrl = $config['loginUrl'];
        $this->loginUrl =  "https://om.teknisa.com/backend/index.php";     //InstanceManager::getInstance()->getParameter('omurl');  
        //Tentei adicionar essa função da linha a cima para não ser uma variavel fixa mas não deu certo, olhar depois. Teoricamente adicionar na linha abaixo
        $this->clientSecret = $integration[0]['clientSecret'];
        $this->clientId = $integration[0]['clientId'];
        
        $nrorgCustom = Utilities::readOAccCustom();
        if(!is_null($nrorgCustom))
            $this->nrorg = $nrorgCustom;
        else
            $this->nrorg = $this->environment->getNrOrgTrab();

        parent::__construct($crudService, $parameterBag);
    }

    /**
     * Procedimento que recebe um filtro e realiza a
     * requisição de atualização dos produtos para a
     * unidade e nível de produto informados. Capta a
     * organização do usuário logado.
     *
     * Envia no objeto response um DataSet com uma
     * mensagem de sucesso ou erro conforme necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function buildAtualizaProduto(DTO\Request\Filter $request, DTO\Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();

        $CDFILIAL    = $filters[0]['value'];
        $CDNVPRODUTO = $filters[1]['value'];
        $NRORG       = $this->environment->getNrOrgTrab();

        try {
            $this->service->atualizaProduto($CDFILIAL, $CDNVPRODUTO, $NRORG);

            $response->addDataSet(new DataSet("success", array("MESSAGE" => "Produtos atualizados com sucesso para a unidade ".$CDFILIAL.".")));
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    /**
     * Procedimento que recebe um filtro e realiza a
     * requisição de validação de endereço para a unidade
     * informada. Capta a organização do usuário logado.
     *
     * Envia no objeto response um DataSet com o
     * resultado ou uma mensagem de erro conforme
     * necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function buildValidationForEndefili(DTO\Request\Filter $request, DTO\Response $response) {
        $validation = null;

        $filters = $request->getFilterCriteria()->getConditions();

        $CDFILIAL = $filters[0]['value'];
        $NRORG    = $this->environment->getNrOrgTrab();

        try {
            $validation = $this->service->validateEndefili($CDFILIAL, $NRORG);

            $response->addDataSet(new DataSet("validation", $validation));
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    /**
     * Procedimento que realiza a requisição de validação
     * da tab de parametrização.
     *
     * Envia no objeto response um DataSet com o
     * resultado ou uma mensagem de erro conforme
     * necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function buildValidationForParamfilipadr(DTO\Request\Filter $request, DTO\Response $response) {
        $validation = null;

        $NRORG  = $this->environment->getNrOrgTrab();

        try {
            $validation = $this->service->validateParamfilipadr($NRORG);

            $response->addDataSet(new DataSet("validation", $validation));
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    /**
     * Procedimento que realiza a requisição de validação
     * da PK de Unidade.
     *
     * Envia no objeto response um DataSet com o
     * resultado ou uma mensagem de erro conforme
     * necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function buildValidationForUnidadePK(DTO\Request\Filter $request, DTO\Response $response) {
        $validation = null;

        $filters = $request->getFilterCriteria()->getConditions();

        $CDFILIAL = $filters[0]['value'];
        $NRORG    = $this->environment->getNrOrgTrab();

        try {
            $validation = $this->service->validateUnidadePK($CDFILIAL, $NRORG);

            $response->addDataSet(new DataSet("validation", $validation));
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    /**
     * Procedimento que deleta o endereço de uma unidade
     * de acordo com os filtros informados.
     *
     * @param Request $request
     * @param Response $response
     */
    public function deleteEndefili(DTO\Request $request, DTO\Response $response) {
        $this->switchDataSource('endefili');

        parent::delete($request, $response);

        $this->switchDataSource('default');
    }

    /**
     * Procedimento que deleta uma Unidade.
     *
     * NOT IMPLEMENTED YET.
     *
     * @param Request $request
     * @param Response $response
     */
    // public function deleteFilial(DTO\Request $request, DTO\Response $response) {
    //     $response->setError(new Error("FILIAL DELETION NOT IMPLEMENTED YET", 500));
    // }

    /**
     * Procedimento que recebe um filtro e realiza a
     * requisição de dados do endereço da unidade de
     * acordo com os filtros informados.
     *
     * Envia no objeto response um DataSet com o
     * resultado ou uma mensagem de erro conforme
     * necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function findEndefili(DTO\Request\Filter $request, DTO\Response $response) {
        $this->switchDataSource('endefili');

        parent::find($request, $response);
        $return = array();
        foreach($response->getDataSets()[0]->getRows() as $row) {
            //loop para reformatar UTF no response
            /*array_walk_recursive($row, function (&$val) {
                if (is_string($val)) {
                    $val = mb_convert_encoding($val, 'UTF-8', 'UTF-8');
                    //$val = $this->convert_from_latin1_to_utf8_recursively($val);
                }
            });*/
            
            $row['CDSENHEMAILF'] = $this->cripto->unCripto($row['CDSENHEMAILF'], 2);
            
            $return[] = $row;
        }
        
        

        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet($this->dataSourceNames['endefili'], $return));

        $this->switchDataSource('default');
    }

    /**
     * Procedimento que recebe um filtro e realiza a
     * requisição de dados da unidade de acordo com os
     * filtros informados.
     *
     * Envia no objeto response um DataSet com o
     * resultado ou uma mensagem de erro conforme
     * necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function findFilial(DTO\Request\Filter $request, DTO\Response $response) {
        $this->switchDataSource('filial');

        parent::find($request, $response);
        $this->switchDataSource('default');
    }

    /**
     * Procedimento que recebe um request e realiza a
     * inserção ou alteração dos dados do endereço da
     * unidade.
     *
     * Envia no objeto response um DataSet com uma
     * mensagem de erro caso necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function saveEndefili(DTO\Request $request, DTO\Response $response) {
        try{
            
            foreach($request->getDataSet()->getRows() as $row) {
                $row = $this->service->validateEndefiliRow($row);
                if(isset($row['CDSENHEMAILF']) && !empty($row['CDSENHEMAILF'])){
                   $row['CDSENHEMAILF'] = $this->service->encrypt($row['CDSENHEMAILF']);
                }
            }
    
            $this->saveGeneric($request, $response, 'endefili');
    
            $this->switchDataSource('default');
        } catch (\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
        
    }

    /**
     * Procedimento que recebe um request e realiza a
     * inserção ou alteração dos dados da unidade.
     *
     * Envia no objeto response um DataSet com uma
     * mensagem de erro caso necessário.
     *
     * Efetua rollback no caso de algum erro.
     *
     * ATENÇÃO! Esse método pressupõe que apenas uma row
     * será recebida! WARNING! AVVERTIMENCO! ADVERTENCIA!
     *
     * @param Filter $request
     * @param Response $response
     */
    public function saveFilial(DTO\Request $request, DTO\Response $response) {
        $this->switchDataSource('filial');
        foreach($request->getDataSet()->getRows() as $row) {
            $row = $this->service->validateUnidadeRow($row);
            $row['NRORG'] = $this->environment->getNrOrgTrab();
            $row['CDPARGERAL'] = '0001';
        }
        $this->entityManager->beginTransaction();
        
        try {
            $this->saveGeneric($request, $response, 'filial', true);
            
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new']) {
                    
                    $row = $this->service->validateUnidadeNewRow($row);
                    
                    /*try {
                        $this->saveGeneric($request, $response, 'filioper', true);
                    } catch(\Exception $e) {
                        //do nothing 
                        //May causes duplicate PK error if client uses new filioper replication trigger. The error can be ignored tho.
                    }*/

                    $this->saveGeneric($request, $response, 'almoxarifado', true);
                    $this->saveGeneric($request, $response, 'localestoque', true);
                    
                    $this->saveGeneric($request, $response, 'paramfilial', true);

                    $NMTABS = array( 
                        'ALMOXARIFADO', 'LOCALESTOQUE', 'PARAMFILIAL',
                        'PARAMPEDIDO', 'PARAMPLANEJA', 'PARCMPSUP', 'PAREXPEDICAO'
                    );

                    foreach ($NMTABS as $NMTABPARAMFILI) {
                        $row['NMTABPARAMFILI'] = $NMTABPARAMFILI;

                    
                        $this->saveGeneric($request, $response, 'filipadr', true);
                    }
                    $this->saveEndefili($request, $response);
                    
                    $this->processCP($row); 
                }
                else{
                    
                    $this->saveGeneric($request, $response, 'paramfilial', true);
                }
                
                if($row['IDPARAMPADRAO'] == 'S'){
                    $row['CDFILIPADRAO' ] = $row['CDFILIAL'];
                    $this->saveGeneric($request, $response, 'paramgeral', true);
                    $allUnidesToReplicate = $this->service->getUnidadesToReplicate();
                    $this->service->callProceduresParamPadrao($allUnidesToReplicate);
                }else{
                    $filiParamPadrao = $this->service->getFiliParamPadrao();
                    if($row['CDFILIAL'] == $filiParamPadrao){
                        $row['CDFILIPADRAO'] = null;
                        $this->saveGeneric($request, $response, 'paramgeral', true);
                        
                    }
                    
                }
            }
            //$this->entityManager->rollback();
            $this->entityManager->commit();
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
            $this->entityManager->rollback();
        }

        $this->switchDataSource('default');
    }   
    
    private function saveGeneric ($request, $response, $dataSourceName, $parentTreatment = false){
        $this->entityManager->beginTransaction();
        try {
            $this->switchDataSource($dataSourceName);
            $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
			//duas linhas a baixo para salvar corretamente a organização relacionada (código e nome) 
			$rows[0]["CDORGRELAC"] = $rows[0]["CDORGRELAC"][0];
			$rows[0]["NMORGRELAC"] = $rows[0]["NMORGRELAC"][0];
			$this->crudService->save($dataSourceName, $rows);
            $this->entityManager->commit(); 
        } catch (\Exception $e){
            if ($parentTreatment) {
                throw $e; 
            }
            $this->entityManager->rollback();
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
        
    }

    /**
     * Procedimento que troca o DataSource ativo do
     * Controller para o equivalente ao label recebido.
     *
     * @param string label
     */
    private function switchDataSource($label) {
        $this->dataSourceName = $this->dataSourceNames[$label];
    }
    
    public function filialCliente(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->service->filialReplicaCliente($row);
    }
    
    public function callProceduresParamPadrao(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->service->callProceduresParamPadrao([$row]);
    }
    
    public function atualizaAbaNfe(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->service->atualizaAbaNfe($row);
    }
    
    public static function convert_from_latin1_to_utf8_recursively($dat)
   {
      if (is_string($dat)) {
         return utf8_encode($dat);
      } elseif (is_array($dat)) {
         $ret = [];
         foreach ($dat as $i => $d) $ret[ $i ] = self::convert_from_latin1_to_utf8_recursively($d);

         return $ret;
      } elseif (is_object($dat)) {
         foreach ($dat as $i => $d) $dat->$i = self::convert_from_latin1_to_utf8_recursively($d);

         return $dat;
      } else {
         return $dat;
      }
   }
   
   public function checkOMacl(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $this->productId = $row['productLoginUrl'];
        $token = $this->getToken();
        $requestType = '/api/findProductElement';
        $dataSetResultPos = 'productElement';
        $keepConnected = false;
        $nr_org_id = $this->getOrganizationInfo($token);
        $data = [
            'requestType' => 'FilterData',
            'filter' => [
                [
                    'name' => 'PRODUCT_ID',
                    'operator' => '=',
                    'value' => $this->productId
                ],
                [
                    'name' => 'NRPRODUTO_ORG_ID',
                    'operator' => '=',
                    'value' => $nr_org_id['NRPRODUTO_ORG_ID'] ?? ''
                ],
                [
                    'name' => 'SHOW_SERVER_INFO',
                    'operator' => '=',
                    'value' => false
                ]
            ]
        ];
        
        $avaliableScreens = Utilities::callWebService($token, $requestType, $data, $dataSetResultPos, $this->clientSecret, $this->loginUrl, $keepConnected);
        $response->addDataSet(new DataSet('data',$avaliableScreens));
   }
   
   public function getToken($uri = null){
        $token = null; 
        $requestType = '/lib_requestToken'; 
        $data = array(
            'CLIENT_ID' => $this->clientId,
            'CLIENT_SECRET' => $this->clientSecret
        ); 
        $dataSetResultPos = 'token'; 
        $privateKey = $this->clientSecret; 
        $uri = $uri ?? $this->loginUrl; 
        $keepConnected = false;
        $data = Utilities::callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $uri, $keepConnected);
        return $data[0];
   }
   
   public function getOrganizationInfo($token){
       
        $requestType = '/api/findOrganizationInformation'; 
        $data = array(
            'requestType' => 'FilterData',
            'filter' => array(
                array('name' => 'PRODUCT_ID', 'operator' => '=', 'value' => $this->productId),
                array('name' => 'SHOW_SERVER_INFO', 'operator' => '=', 'value' => false)
            )
        ); 
        $dataSetResultPos = 'organizationInformation'; 
        $privateKey = $this->clientSecret; 
        $uri = $this->loginUrl; 
        $keepConnected = false;
        
        $organizationInformation = Utilities::callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $uri, $keepConnected);
        $organization = current(array_filter($organizationInformation,function($org){
            return $org['NRORG'] == $this->nrorg;
        }));

        return $organization;
   }
   
    public function sendEmailToSuppliers($row) {
        $row = $row->getRow();
      
        return  $this->service->sendEmailToSuppliers($row);
    }
    
   public function processCP($row){
        $this->productId = $row['productLoginUrl'];
        $requestType = '/api/saveCustomizationFiles';
        $dataSetResultPos = 'saveCustomizationFiles';
        $keepConnected = false;
        $tokenCP = $this->getToken($this->cpUrl);
        $privateKey = $this->clientSecret; 
        $data = array(
          "requestType" => "DataSet",
          "dataset" => array( array(
            'NRORG' => $this->nrorg,
            'CDLOCALUSO' => $row['CDFILIAL'],
            'NMFANTLUSO' => substr($row['NMFILIAL'], 0, 50),                                                        
            'NMRAZSOCLUSO' => substr($row['NMRAZSOCFILI'], 0, 50),         
            'IDTPIJURLUSO' => $row['IDTPIJURFILI'],
            'NRINSJURLUSO' => substr($row['MSKINSJURFILI'], 0, 20),
            'NRINSESTLUSO' => substr($row['CDINSCESTA'], 0, 16),
            'NRINSMUNILUSO' => substr($row['CDINSCMUNI'], 0, 20),
            'NMCONTLUSO' => substr($this->environment->getCurrentUserName(), 0, 50),
            'NMDEPTOLUSO' => substr($row['NMRAZSOCEMPR'], 0, 50),
            'DSENDELUSO' => $row['DSENDEFILI'] ? substr($row['DSENDEFILI'], 0, 50) : 'Sem Endereço', //alt+255
            'NMBAIRLUSO' => $row['NMBAIRRO'] ? substr($row['NMBAIRRO'], 0, 30) : 'Sem Bairro',                 
            'NRCEPLUSO' => isset($row['NRCEPFILI']) ? str_replace('.', '', substr($row['NRCEPFILI'], 0, 15)) : '00000-000',           
            'CDPAIS' => $row['CDPAIS'],
            'SGESTADO' => substr($row['SGESTADO'], 0, 2),                                                                         
            'CDMUNICIPIO' => substr($row['CDMUNICIPIO'], 0, 4), //este campo pode dar problema.                                  
            'DSEMAILCOMLU' => substr($this->environment->getCurrentEmail(), 0, 50),                                    
            'CDLINPCLIE' => $row['productLines'] ?? [],
            'IDMODPROC' => 'F',
            'IDLOCAPROC' => 'F'
          ))
        ); 
        try{
            $result = Utilities::callWebService($tokenCP, $requestType, $data, $dataSetResultPos, $privateKey, $this->cpUrl, $keepConnected);    
            return $result;
        }catch(\Exception $e){
            throw $e;
        }
   }
   
   public function getLinhasProdutos(DTO\Request $request, DTO\Response $response){
        $tokenCP = $this->getToken($this->cpUrl);
        $requestType = '/api/findUseLicense';
        $dataSetResultPos = 'findUseLicense'; 
        $privateKey = $this->clientSecret; 
        $keepConnected = false;
        $data = array(
            'requestType' => 'FilterData',
            'filter' => array(
                array('name' => 'NRORG', 'operator' => '=', 'value' => $this->nrorg),
            )
        );
        
        $result = Utilities::callWebService($tokenCP, $requestType, $data, $dataSetResultPos, $privateKey, $this->cpUrl, $keepConnected);
        if(is_null($result))
            $result = [];
        $response->addDataSet(new DataSet('getLinhasProdutos',$result));
   }
   
   public function getTeknisaCountry(DTO\Request $request, DTO\Response $response){
        $nrorg = '20';
        $filter = $request->getFilterCriteria()->getConditions();
        foreach($filter as &$valor){
            $valor['name'] = $valor['columnName'];
            unset($valor['columnName']);
        }
        array_push($filter, array('name' => 'NRORG', 'operator' => '=', 'value' => $nrorg));
        $token = $this->getToken($this->cpUrl);
        $requestType = '/api/findCountry';
        $data = array(
            'requestType' => 'FilterData',
            'filter' => $filter
        ); 
        $dataSetResultPos = 'findCountry'; 
        $privateKey = $this->clientSecret;
        $url = $this->cpUrl;
        
        $result = Utilities::callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url);
        $response->addDataSet(new DataSet('getTeknisaCountry',$result));
   }
   
   public function getTeknisaState(DTO\Request $request, DTO\Response $response){
        $nrorg = '20';
        $filter = $request->getFilterCriteria()->getConditions();
        foreach($filter as &$valor){
            $valor['name'] = $valor['columnName'];
            unset($valor['columnName']);
        }
        
        $token = $this->getToken($this->cpUrl);
        $requestType = '/api/findState';
        $data = array(
            'requestType' => 'FilterData',
            'filter' => $filter
        ); 
        $dataSetResultPos = 'findState'; 
        $privateKey = $this->clientSecret;
        $url = $this->cpUrl;
        $result = Utilities::callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url);
        
        $response->addDataSet(new DataSet('getTeknisaState',$result));
   }
   
   public function getTeknisaCity(DTO\Request $request, DTO\Response $response){
        $nrorg = '20';
        $filter = $request->getFilterCriteria()->getConditions();
        foreach($filter as &$valor){
            $valor['name'] = $valor['columnName'];
            unset($valor['columnName']);
        }
        
        $token = $this->getToken($this->cpUrl);
        $requestType = '/api/findCity';
        $data = array(
            'requestType' => 'FilterData',
            'filter' => $filter
        ); 
        $dataSetResultPos = 'findCity'; 
        $privateKey = $this->clientSecret;
        $url = $this->cpUrl;
        
        $result = Utilities::callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url);
        
        $response->addDataSet(new DataSet('getTeknisaCity',$result));
   }
    
}
