<?php
namespace Teknisa\CADEST\Controllers;

use Teknisa\CADEST\Services\ConnectionService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;

/**
 * Class QueryExecute
 *
 * @package Controller
 */
class QueryExecute extends Simple {

    /** @var ConnectionService */
    protected $connectionService;
    /** @var array Used to translate the query type into a information to the user after execution */
    protected $translate = array('UPDATE' => "updated", 'INSERT' => "inserted", 'DELETE' => "deleted");

    public function __construct(ConnectionService $connectionService) {
        $this->connectionService = $connectionService;
    }

    /**
     * @param int $count
     *
     * @return string
     */
    protected function buildCountReturnString($count) {
        return $count. " affected row(s).";
    }

    /**
     * Runs a query in database
     *
     * @param DTO\Request\Row $request, DTO\Response $response
     *
     * @return DataSet Dataset came from query executed
     *
     * @throws exception When user types a invalid query or any problem happens with connection
     *
     */
    public function queryRun(DTO\Request\Row $request, DTO\Response $response) {


        $row = $request->getRow();
        
        try {
            if(is_array($row["query"])){
                $connectionReturn = $this->connectionService->executeQueryArrayInConnection($row);
            } else{
                $connectionReturn = $this->connectionService->executeQueryInConnection($row);
            }

            if ($connectionReturn["type"] == ConnectionService::RETURN_TYPE_DATA_SET) {
                $response->addDataSet(new DataSet('result', $connectionReturn["rows"]));
            } else if($connectionReturn["type"] == ConnectionService::RETURN_TYPE_AFFECTED_ROWS) {
                $response->addDataSet(new DataSet('result', $connectionReturn["count"]));
            } else if($connectionReturn["type"] == ConnectionService::RETURN_TYPE_ERROR) {
                $response->addDataSet(new DataSet('result', $connectionReturn["error"]));
            } else if($connectionReturn["type"] == ConnectionService::RETURN_TYPE_MESSAGE) {
                $response->addDataSet(new DataSet('result', $connectionReturn["message"]));
            } else if($connectionReturn["type"] == ConnectionService::RETURN_TYPE_DATA_SET_ARRAY) {
                $response->addDataSet(new DataSet('result', $connectionReturn["logs"]));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 0));
        }
    }
}